package com.googlecode.alfresco.repository.node.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.alfresco.model.ContentModel;
import org.alfresco.service.cmr.repository.ChildAssociationRef;
import org.alfresco.service.cmr.repository.NodeRef;
import org.alfresco.service.cmr.repository.NodeService;
import org.alfresco.service.cmr.repository.StoreRef;
import org.alfresco.service.cmr.search.CategoryService;
import org.alfresco.service.namespace.QName;
import org.alfresco.service.namespace.RegexQNamePattern;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.util.Assert;

import com.googlecode.alfresco.repository.node.CategoryHelper;

public class CategoryHelperImpl implements CategoryHelper {

	private CategoryService categoryService;

	private NodeService nodeService;

	private boolean caseSensitiveNameMatching = false;

	private List<QName> propertyNames = Arrays.asList(ContentModel.PROP_NAME);

	@Required
	public void setCategoryService(final CategoryService categoryService) {
		Assert.notNull(categoryService);
		this.categoryService = categoryService;
	}

	protected CategoryService getCategoryService() {
		return categoryService;
	}

	@Required
	public void setNodeService(final NodeService nodeService) {
		Assert.notNull(nodeService);
		this.nodeService = nodeService;
	}

	protected NodeService getNodeService() {
		return nodeService;
	}

	public void setPropertyNames(final List<QName> propertyNames) {
		Assert.notEmpty(propertyNames);
		this.propertyNames = propertyNames;
	}

	protected List<QName> getPropertyNames() {
		return propertyNames;
	}

	public void setCaseSensitiveNameMatching(final boolean caseSensitiveNameMatching) {
		this.caseSensitiveNameMatching = caseSensitiveNameMatching;
	}

	public boolean isCaseSensitiveNameMatching() {
		return caseSensitiveNameMatching;
	}

	@Override
	public NodeRef getCategoryNodeRef(final StoreRef storeRef, final QName aspectName, final String... names) {
		Assert.notNull(storeRef, "StoreRef cannot be null.");
		Assert.notNull(aspectName, "Aspect cannot be null.");
		Assert.notEmpty(names, "Names cannot be empty");
		final List<String> namesList = new ArrayList<String>(Arrays.asList(names));
		for (final ChildAssociationRef rootCategory : getCategoryService().getRootCategories(storeRef, aspectName)) {
			if (nameMatches(rootCategory.getChildRef(), namesList.get(0))) {
				return findCategory(rootCategory.getChildRef(), namesList.subList(1, namesList.size()));
			}
		}
		return null;
	}

	@Override
	public NodeRef getCategoryNodeRef(final String... names) {
		return getCategoryNodeRef(StoreRef.STORE_REF_WORKSPACE_SPACESSTORE, ContentModel.ASPECT_GEN_CLASSIFIABLE, names);
	}

	@Override
	public NodeRef getCategoryNodeRef(final QName aspectName, final String... names) {
		return getCategoryNodeRef(StoreRef.STORE_REF_WORKSPACE_SPACESSTORE, aspectName, names);
	}

	protected NodeRef findCategory(final NodeRef categoryNodeRef, final List<String> names) {
		Assert.notNull(categoryNodeRef);
		Assert.notEmpty(names);
		final String name = names.remove(0);
		for (final ChildAssociationRef childAssociationRef : getNodeService().getChildAssocs(categoryNodeRef,
				ContentModel.ASSOC_SUBCATEGORIES, RegexQNamePattern.MATCH_ALL, true)) {
			final NodeRef childNodeRef = childAssociationRef.getChildRef();
			if (nameMatches(childNodeRef, name)) {
				if (names.isEmpty()) {
					return childNodeRef;
				} else {
					return findCategory(childNodeRef, names);
				}
			}
		}
		return null;
	}

	protected boolean nameMatches(final NodeRef nodeRef, final String name) {
		Assert.notNull(nodeRef);
		Assert.hasText(name);
		for (final QName propertyName : getPropertyNames()) {
			final String propertyValue = (String) getNodeService().getProperty(nodeRef, propertyName);
			if (isCaseSensitiveNameMatching()) {
				if (name.equals(propertyValue)) {
					return true;
				}
			} else {
				if (name.equalsIgnoreCase(propertyValue)) {
					return true;
				}
			}
		}
		return false;
	}

}
