package com.googlecode.alfresco.spring;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * Spring {@link FactoryBean} for creating a child {@link ApplicationContext}.
 *
 * @author Laurens Fridael
 *
 */
public class ChildApplicationContextFactoryBean implements FactoryBean<ClassPathXmlApplicationContext>,
		ApplicationContextAware {

	private String[] configLocations;

	private ApplicationContext parentApplicationContext;

	private ClassPathXmlApplicationContext childApplicationContext;

	public void setConfigLocations(final String[] configLocations) {
		this.configLocations = configLocations;
	}

	protected String[] getConfigLocations() {
		return configLocations;
	}

	@Override
	public void setApplicationContext(final ApplicationContext applicationContext) throws BeansException {
		this.parentApplicationContext = applicationContext;
	}

	protected ApplicationContext getParentApplicationContext() {
		return parentApplicationContext;
	}

	@Override
	public boolean isSingleton() {
		return true;
	}

	@Override
	public Class<? extends ClassPathXmlApplicationContext> getObjectType() {
		return ClassPathXmlApplicationContext.class;
	}

	@Override
	public ClassPathXmlApplicationContext getObject() {
		if (childApplicationContext == null) {
			childApplicationContext = createOsgiContainerApplicationContext();
		}
		return childApplicationContext;
	}

	protected ClassPathXmlApplicationContext createOsgiContainerApplicationContext() {
		return new ClassPathXmlApplicationContext(getConfigLocations(), getParentApplicationContext());
	}

	public void destroy() {
		closeOsgiContainerApplicationContext();
	}

	protected void closeOsgiContainerApplicationContext() {
		if (childApplicationContext != null) {
			childApplicationContext.close();
		}
	}

}
